<?php
/*
Plugin Name: InsaneJournal Crossposter
Plugin URI: http://code.google.com/p/ljxp/
Description: Automatically copies all posts to InsaneJournal. Editing or deleting a post will be replicated as well. This plugin was inspired by <a href="http://blog.mytechaid.com/">Scott Buchanan's</a> <a href="http://blog.mytechaid.com/archives/2005/01/10/xanga-crossposter/">Xanga Crossposter</a>.
Version: 2.0
Author: Evan Broder (modified for insanejournal by rusty_halo)
Author URI: http://ebroder.net/

	Copyright (c) 2007 Evan Broder

	Permission is hereby granted, free of charge, to any person obtaining a
	copy of this software and associated documentation files (the "Software"),
	to deal in the Software without restriction, including without limitation
	the rights to use, copy, modify, merge, publish, distribute, sublicense,
	and/or sell copies of the Software, and to permit persons to whom the
	Software is furnished to do so, subject to the following conditions:

	The above copyright notice and this permission notice shall be included in
	all copies or substantial portions of the Software.

	THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
	IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
	FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
	AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
	LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
	FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
	DEALINGS IN THE SOFTWARE.
*/

define('IJXP_DOMAIN', '/ijxp/lang/ijxp');
load_plugin_textdomain(IJXP_DOMAIN);

require_once(ABSPATH . '/wp-includes/class-IXR.php');
if(version_compare($wp_version, "2.1", "<")) {
	require_once(ABSPATH . '/wp-includes/template-functions-links.php');
}

// Create the IJXP Options Page
function ijxp_add_pages() {
	add_options_page("InsaneJournal", "InsaneJournal", 6, __FILE__, 'ijxp_display_options');
}

// Display the options page
function ijxp_display_options() {
	global $wpdb;

	// Just in case they don't exist, create the options
	add_option('ijxp_host');
	add_option('ijxp_username');
	add_option('ijxp_password');
	add_option('ijxp_custom_name_on');
	add_option('ijxp_custom_name');
	add_option('ijxp_privacy');
	add_option('ijxp_comments');
	add_option('ijxp_tag');
	add_option('ijxp_more');
	add_option('ijxp_community');
	add_option('ijxp_skip_cats');
	add_option('ijxp_header_loc');
	add_option('ijxp_custom_header');

	// Retrieve these for the form
	$old_host = stripslashes(get_option('ijxp_host'));
	$old_username = stripslashes(get_option('ijxp_username'));
	$old_name = stripslashes(get_option('ijxp_custom_name'));
	$old_name_on = get_option('ijxp_custom_name_on');
	$old_privacy = get_option('ijxp_privacy');
	$old_comments = get_option('ijxp_comments');
	$old_tag = get_option('ijxp_tag');
	$old_more = get_option('ijxp_more');
	$old_community = stripslashes(get_option('ijxp_community'));
	// Categories to be crossposted are stored not stored; categories to be
	// "skipped" are instead. It's inverted so that new categories can be
	// assumed to be in the clear
	$old_skip_cats = get_option('ijxp_skip_cats');
	$old_header_loc = get_option('ijxp_header_loc');
	$old_custom_header = stripslashes(get_option('ijxp_custom_header'));

	// host should default to IJ
	if("" == $old_host) {
		// This sets up a default value. If we don't store it, the default val
		// will never get stored to the database
		update_option('ijxp_host', 'www.insanejournal.com');
		$old_host = "www.insanejournal.com";
	}

	// I think that we should default to just using the name of the blog, so
	// let's set it - same reason as above
	if("" == $old_name_on) {
		update_option('ijxp_custom_name_on', '0');
		$old_name_on = "0";
	}

	// We're going to default to public posts - just because I say so
	if("" == $old_privacy) {
		update_option('ijxp_privacy', 'public');
		$old_privacy = "public";
	}

	// Defaulting to no comments on IJ - makes more sense
	if("" == $old_comments) {
		update_option('ijxp_comments', '0');
		$old_comments = '0';
	}

	// Default to allowing tags - only in strange i18n situations would you
	// want them disabled
	if("" == $old_tag) {
		update_option('ijxp_tag', '1');
		$old_tag = '1';
	}

	// The default option is to link back to the original site if there is a
	// <!--more--> tag
	if("" == $old_more) {
		update_option('ijxp_more', 'link');
		$old_more = 'link';
	}

	if("" == $old_skip_cats) {
		update_option('ijxp_skip_cats', array());
		$old_skip_cats = array();
	}

	// 0 means top, 1 means bottom
	if("" == $old_header_loc) {
		update_option('ijxp_header_loc', 0);
		$old_header_loc = 0;
	}

	// If we're handling a submission, save the data
	if(isset($_REQUEST['update_ij_options']) || isset($_REQUEST['crosspost_all'])) {
		// If certain values get changed, we want to recrosspost all entries,
		// so just in case, grab a list of all entries that have been
		// crossposted
		$repost_ids = $wpdb->get_col("SELECT post_id FROM $wpdb->postmeta WHERE meta_key='ijID'");
		// This is just a tracking variable. If changes are made to multiple
		// settings that would each require reposting everything, we don't want
		// to spam IJ's servers, so we just keep track of whether or not we
		// need to do a massive update at the end. This isn't necessary with
		// deletions, as the function will just give up after the first time
		$need_update = 0;

		// Avoiding useless queries - confirming that the value changed
		if($old_host != $_REQUEST['host']) {
			// So that we don't have crossposted residue left behind on the old
			// server, get rid of all posts. This also eliminates potential
			// problems when any crosspotsed entry is edited - before, IJXP
			// would have tried to edit a nonexistent entry on the new server
			ijxp_delete_all($repost_ids);
			update_option('ijxp_host', $_REQUEST['host']);
			// So that the new value shows up in the form
			$old_host = $_REQUEST['host'];
			// Then repost everything
			$need_update = 1;
		}

		if($old_username != $_REQUEST['username']) {
			ijxp_delete_all($repost_ids);
			update_option('ijxp_username', $_REQUEST['username']);
			$old_username = $_REQUEST['username'];
			$need_update = 1;
		}

		if($old_name_on != $_REQUEST['custom_name_on']) {
			update_option('ijxp_custom_name_on', $_REQUEST['custom_name_on']);
			$old_name_on = $_REQUEST['custom_name_on'];
			$need_update = 1;
		}

		if($old_name != $_REQUEST['custom_name']) {
			update_option('ijxp_custom_name', $_REQUEST['custom_name']);
			$old_name = $_REQUEST['custom_name'];
			if($old_name_on) {
				$need_update = 1;
			}
		}

		if($old_privacy != $_REQUEST['privacy']) {
			update_option('ijxp_privacy', $_REQUEST['privacy']);
			$old_privacy = $_REQUEST['privacy'];
			$need_update = 1;
		}

		if($old_comments != $_REQUEST['comments']) {
			update_option('ijxp_comments', $_REQUEST['comments']);
			$old_comments = $_REQUEST['comments'];
			$need_update = 1;
			ijxp_post_all($repost_ids);
		}

		if($old_tag != $_REQUEST['tag']) {
			update_option('ijxp_tag', $_REQUEST['tag']);
			$old_tag = $_REQUEST['tag'];
		}

		if($old_more != $_REQUEST['more']) {
			update_option('ijxp_more', $_REQUEST['more']);
			$old_more = $_REQUEST['more'];
			$need_update = 1;
		}

		if($old_community != $_REQUEST['community']) {
			ijxp_delete_all($repost_ids);
			update_option('ijxp_community', $_REQUEST['community']);
			$old_community = $_REQUEST['community'];
			$need_update = 1;
		}

		if($old_header_loc != $_REQUEST['header_loc']) {
			update_option('ijxp_header_loc', $_REQUEST['header_loc']);
			$old_header_loc = $_REQUEST['header_loc'];
		}

		if($old_custom_header != $_REQUEST['custom_header']) {
			update_option('ijxp_custom_header', $_REQUEST['custom_header']);
			$old_custom_header = $_REQUEST['custom_header'];
		}

		sort($old_skip_cats);
		$new_skip_cats = array_diff(get_all_category_ids(), (array)$_REQUEST['post_category']);
		sort($new_skip_cats);
		if($old_skip_cats != $new_skip_cats) {
			update_option('ijxp_skip_cats', $new_skip_cats);
			$old_skip_cats = $new_skip_cats;
		}

		// If a password value is entered, md5 it for security and store to the
		// database
		// IJ challenge authentication works with only knowing the md5 of the
		// password
		if($_REQUEST['password'] != "") {
			update_option('ijxp_password', md5($_REQUEST['password']));
		}

		if($need_update && isset($_REQUEST['update_ij_options'])) {
			@set_time_limit(0);
			ijxp_post_all($repost_ids);
		}

		if(isset($_REQUEST['crosspost_all'])) {
			@set_time_limit(0);
			ijxp_post_all($wpdb->get_col("SELECT ID FROM $wpdb->posts WHERE post_status='publish' AND post_type='post'"));
		}

		// Copied from another options page
		echo '<div id="message" class="updated fade"><p><strong>';
		_e('Options saved.', IJXP_DOMAIN);
		echo '</strong></p></div>';
	}

	// And, finally, output the form
	// May add some Javascript to disable the custom_name field later - don't
	// feel like it now, though
?>
<div class="wrap">
	<form method="post" action="<?php echo $_SERVER['REQUEST_URI']; ?>">
		<h2><?php _e('InsaneJournal Crossposter Options', IJXP_DOMAIN); ?></h2>
		<table width="100%" cellspacing="2" cellpadding="5" class="editform">
			<tr valign="top">
				<th width="33%" scope="row"><?php _e('InsaneJournal-compliant host:', IJXP_DOMAIN) ?></th>
				<td><input name="host" type="text" id="host" value="<?php echo htmlentities($old_host); ?>" size="40" /><br />
				<?php

				_e('If you are using a InsaneJournal-compliant site other than InsaneJournal (like DeadJournal), enter the domain name here. InsaneJournal users can use the default value', IJXP_DOMAIN);

				?>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php _e('IJ Username', IJXP_DOMAIN); ?></th>
				<td><input name="username" type="text" id="username" value="<?php echo htmlentities($old_username); ?>" size="40" /></td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php _e('IJ Password', IJXP_DOMAIN); ?></th>
				<td><input name="password" type="password" id="password" value="" size="40" /><br />
				<?php

				_e('Only enter a value if you wish to change the stored password. Leaving this field blank will not erase any passwords already stored.', IJXP_DOMAIN);

				?>
				</td>
			</tr>
			<tr valign="top">
				<th scope="row"><?php _e('Community', IJXP_DOMAIN); ?></th>
				<td><input name="community" type="text" id="community" value="<?php echo htmlentities($old_community); ?>" size="40" /><br />
				<?php

				_e("If you wish your posts to be copied to a community, enter the community name here. Leaving this space blank will copy the posts to the specified user's journal instead", IJXP_DOMAIN);

				?>
				</td>
			</tr>
		</table>
		<fieldset class="options">
			<legend><?php _e('Blog Header', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Crosspost header/footer location', IJXP_DOMAIN); ?></th>
					<td><label><input name="header_loc" type="radio" value="0" <?php
					if(0 == $old_header_loc) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Top of post', IJXP_DOMAIN); ?></label><br />
					<label><input name="header_loc" type="radio" value="1" <?php
					if(1 == $old_header_loc) {
						echo 'checked="checked" ';
					}
					?>/> <? _e('Bottom of post', IJXP_DOMAIN); ?></label></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Set blog name for crosspost header/footer', IJXP_DOMAIN); ?></th>
					<td><label><input name="custom_name_on" type="radio" value="0" <?php
					if(0 == $old_name_on) {
						echo 'checked="checked" ';
					}
					?>/> <?php printf(__('Use the title of your blog (%s)', IJXP_DOMAIN), get_settings('blogname')); ?></label><br />
					<label><input name="custom_name_on" type="radio" value="1" <?php
					if(1 == $old_name_on) {
						echo 'checked="checked" ';
					}
					?>/> <? _e('Use a custom title', IJXP_DOMAIN); ?></label></td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Custom blog title', IJXP_DOMAIN); ?></th>
					<td><input name="custom_name" type="text" id="custom_name" value="<?php echo htmlentities($old_name); ?>" size="40" /><br />
					<?php

					_e('If you chose to use a custom title above, enter the title here. This will be used in the header which links back to this site at the top of each post on the InsaneJournal.', IJXP_DOMAIN);

					?>
					</td>
				</tr>
				<tr valign="top">
					<th scope="row"><?php _e('Custom crosspost header/footer', IJXP_DOMAIN); ?></th>
					<td><textarea name="custom_header" id="custom_name" rows="3" cols="40"><?php echo htmlentities($old_custom_header); ?></textarea><br />
					<?php

					_e("If you wish to use IJXP's dynamically generated post header/footer, you can ignore this setting. If you don't like the default crosspost header/footer, specify your own here. For flexibility, you can choose from a series of case-sensitive substitution strings, listed below:", IJXP_DOMAIN);

					?>
					<dl>
						<dt>[blog_name]</dt>
						<dd><?php _e('The title of your blog, as specified above', IJXP_DOMAIN); ?></dd>

						<dt>[blog_link]</dt>
						<dd><?php _e("The URL of your blog's homepage", IJXP_DOMAIN); ?></dd>

						<dt>[permalink]</dt>
						<dd><?php _e('A permanent URL to the post being crossposted', IJXP_DOMAIN); ?></dd>

						<dt>[comments_link]</dt>
						<dd><?php _e('The URL for comments. Generally this is the permalink URL with #comments on the end', IJXP_DOMAIN); ?></dd>
					</dl>
					</td>
			</table>
		</fieldset>
		<fieldset class="options">
			<legend><?php _e('Post Privacy', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Privacy level for all posts to InsaneJournal', IJXP_DOMAIN); ?></th>
					<td><label><input name="privacy" type="radio" value="public" <?php
					if("public" == $old_privacy) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Public', IJXP_DOMAIN); ?></label><br />
					<label><input name="privacy" type="radio" value="private" <?php
					if("private" == $old_privacy) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Private', IJXP_DOMAIN); ?></label><br />
					<label><input name="privacy" type="radio" value="friends" <?php
					if("friends" == $old_privacy) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Friends only', IJXP_DOMAIN); ?></label><br />
				</tr>
			</table>
		</fieldset>
		<fieldset class="options">
			<legend><?php _e('InsaneJournal Comments', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Should comments be allowed on InsaneJournal?', IJXP_DOMAIN); ?></th>
					<td><label><input name="comments" type="radio" value="0" <?php
					if(0 == $old_comments) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Require users to comment on WordPress', IJXP_DOMAIN); ?></label><br />
					<label><input name="comments" type="radio" value="1" <?php
					if("1" == $old_comments) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Allow comments on InsaneJournal', IJXP_DOMAIN); ?></label><br />
				</tr>
			</table>
		</fieldset>
		<fieldset class="options">
			<legend><?php _e('InsaneJournal Tags', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33% scope="row"><?php _e('Tag entries on InsaneJournal?', IJXP_DOMAIN); ?></th>
					<td><label><input name="tag" type="radio" value="1" <?php
					if(1 == $old_tag) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Tag InsaneJournal entries with WordPress categories', IJXP_DOMAIN); ?></label><br />
					<label><input name="tag" type="radio" value="0" <?php
					if(0 == $old_tag) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Do not tag InsaneJournal entries', IJXP_DOMAIN); ?></label><br />
					<?php

					_e('You may with to disable this feature if you are posting in an alphabet other than the Roman alphabet. InsaneJournal does not seem to support non-Roman alphabets in tag names.', IJXP_DOMAIN);

					?>
					</td>
				</tr>
			</table>
		</fieldset>
		<fieldset class="options">
			<legend><?php _e('Handling of &lt;!--More--&gt;', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('How should IJXP handle More tags?', IJXP_DOMAIN); ?></th>
					<td><label><input name="more" type="radio" value="link" <?php
					if("link" == $old_more) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Link back to WordPress', IJXP_DOMAIN); ?></label><br />
					<label><input name="more" type="radio" value="lj-cut" <?php
					if("lj-cut" == $old_more) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Use an lj-cut', IJXP_DOMAIN); ?></label><br />
					<label><input name="more" type="radio" value="copy" <?php
					if("copy" == $old_more) {
						echo 'checked="checked" ';
					}
					?>/> <?php _e('Copy the entire entry to InsaneJournal', IJXP_DOMAIN); ?></label><br />
				</tr>
			</table>
		</fieldset>
		<fieldset class="options">
			<legend><?php _e('Category Selection', IJXP_DOMAIN); ?></legend>
			<table width="100%" cellspacing="2" cellpadding="5" class="editform">
				<tr valign="top">
					<th width="33%" scope="row"><?php _e('Select which categories should be crossposted', IJXP_DOMAIN); ?></th>
					<td>
					<?php

					write_nested_categories(ijxp_cat_select(get_nested_categories(), $old_skip_cats));

					?><br />
					<?php

					_e('Any post that has <em>at least one</em> of the above categories selected will be crossposted.');

					?>
					</td>
				</tr>
			</table>
		</fieldset>
		<p class="submit">
			<input type="submit" name="crosspost_all" value="<?php _e('Update Options and Crosspost All WordPress entries', IJXP_DOMAIN); ?>" />
			<input type="submit" name="update_ij_options" value="<?php _e('Update Options'); ?>" style="font-weight: bold;" />
		</p>
	</form>
</div>
<?php
}

function ijxp_cat_select($cats, $selected_cats) {
	foreach((array)$cats as $key=>$cat) {
		$cats[$key]['checked'] = !in_array($cat['cat_ID'], $selected_cats);
		$cats[$key]['children'] = ijxp_cat_select($cat['children'], $selected_cats);
	}
	return $cats;
}

function ijxp_post($post_id) {
	global $wpdb;

	// If the post was manually set to not be crossposted, give up now
	if(get_post_meta($post_id, 'no_ij', true)) {
		return $post_id;
	}

	// Get the relevent info out of the database
	$host = stripslashes(get_option('ijxp_host'));
	$user = stripslashes(get_option('ijxp_username'));
	$pass = get_option('ijxp_password');
	$custom_name_on = get_option('ijxp_custom_name_on');
	$custom_name = stripslashes(get_option('ijxp_custom_name'));
	$privacy = get_option('ijxp_privacy');
	$comments = get_option('ijxp_comments');
	$tag = get_option('ijxp_tag');
	$more = get_option('ijxp_more');
	$community = stripslashes(get_option('ijxp_community'));
	$skip_cats = get_option('ijxp_skip_cats');
	$copy_cats = array_diff(get_all_category_ids(), $skip_cats);
	$header_loc = get_option('ijxp_header_loc');
	$custom_header = stripslashes(get_option('ijxp_custom_header'));

	// Override the default options if done for this specific post
	// The subtraction bit just converts the values for the post_meta to the
	// bool for the $comments var
	if(0 != get_post_meta($post_id, 'ijxp_comments', true)) {
		$comments = 2 - get_post_meta($post_id, 'ijxp_comments', true);
	}

	if("" != get_post_meta($post_id, 'ijxp_privacy', true)) {
		$privacy = get_post_meta($post_id, 'ijxp_privacy', true);
	}

	// If the post shows up in the forbidden category list and it has been
	// crossposted before (so the forbidden category list must have changed),
	// delete the post. Otherwise, just give up now
	$do_crosspost = 0;
	foreach(wp_get_post_cats(1, $post_id) as $cat) {
		if(in_array($cat, $copy_cats)) {
			$do_crosspost = 1;
		}
	}
	if(!$do_crosspost) {
		return ijxp_delete($post_id);
	}

	// And create our connection
	$client = new IXR_Client($host, '/interface/xmlrpc');

	// Get the challenge string
	// Using challenge for the most security. Allows pwd hash to be stored
	// instead of pwd
	if (!$client->query('LJ.XMLRPC.getchallenge')) {
		wp_die('Something went wrong - '.$client->getErrorCode().' : '.$client->getErrorMessage());
	}

	// And retrieve the challenge string
	$response = $client->getResponse();
	$challenge = $response['challenge'];

	$post = & get_post($post_id);

	// Insert the name of the page we're linking back to based on the options set
	if(!$custom_name_on) {
		$blogName .= get_option("blogname");
	}
	else {
		$blogName .= $custom_name;
	}

	if('' == $custom_header) {
		$postHeader = '<p style="border: 1px solid black; padding: 3px;"><b>';

		// If the post is not password protected, follow standard procedure
		if(!$post->post_password) {
			$postHeader .= __('Originally published at', IJXP_DOMAIN);
			$postHeader .= ' <a href="'.get_permalink($post_id).'">';
			$postHeader .= $blogName;
			$postHeader .= '</a>.';
		}
		// If the post is password protected, put up a special message
		else {
			$postHeader .= __('This post is password protected. You can read it at', IJXP_DOMAIN);
			$postHeader .= ' <a href="'.get_permalink($post_id).'">';
			$postHeader .= $blogName;
			$postHeader .= '</a>, ';
			$postHeader .= __('where it was originally posted', IJXP_DOMAIN);
			$postHeader .= '.';
		}

		// Depending on whether comments or allowed or not, alter the header
		// appropriately
		if($comments) {
			$postHeader .= sprintf(__(' You can comment here or <a href="%s#comments">there</a>.', IJXP_DOMAIN), get_permalink($post_id));
		}
		else {
			$postHeader .= sprintf(__(' Please leave any <a href="%s#comments">comments</a> there.', IJXP_DOMAIN), get_permalink($post_id));
		}

		$postHeader .= '</b></p>';
	}
	else {
		$postHeader = $custom_header;

		$find = array('[blog_name]', '[blog_link]', '[permalink]', '[comments_link]');
		$replace = array($blogName, get_settings('home'), get_permalink($post_id), get_permalink($post_id).'#comments');
		$postHeader = str_replace($find, $replace, $postHeader);
	}

	// $the_event will eventually be passed to the IJ XML-RPC server.
	$the_event = "";

	// and if the post isn't password protected, we need to put together the
	// actual post
	if(!$post->post_password) {
		// and if there's no <!--more--> tag, we can spit it out and go on our
		// merry way
		if(strpos($post->post_content, "<!--more-->") === false) {
			$the_event .= apply_filters('the_content', $post->post_content);
		}
		else {
			$content = explode("<!--more-->", $post->post_content, 2);
			$the_event .= apply_filters('the_content', $content[0]);
			switch($more) {
			case "copy":
				$the_event .= apply_filters('the_content', $content[1]);
				break;
			case "link":
				$the_event .= sprintf('<p><a href="%s#more-%s">', get_permalink($post_id), $post_id) .
					__('Read the rest of this entry &raquo;', IJXP_DOMAIN) .
					'</a></p>';
				break;
			case "lj-cut":
				$the_event .= '<lj-cut text="' .
					__('Read the rest of this entry &amp;raquo;', IJXP_DOMAIN) .
					'">' . apply_filters('the_content', $content[1]) . '</lj-cut>';
				break;
			}
		}
	}

	// Either prepend or append the header to $the_event, depending on the
	// config setting
	// Remember that 0 is at the top, 1 at the bottom
	if($header_loc) {
		$the_event .= $postHeader;
	}
	else {
		$the_event = $postHeader.$the_event;
	}

	// Retrieve the categories that the post is marked as - for IJ tagging
	$cats = wp_get_post_cats('', $post_id);
	// I need them in an array for my next trick to work
	if(!is_array($cats)) {
		$cats = array($cats);
	}
	// Convert the category IDs of all categories to their text names
	$cat_names = array_map("get_cat_name", $cats);
	// Turn them into a comma-seperated list for the API
	$cat_string = implode(", ", $cat_names);

	// Get the most recent post (to see if this is it - it it's not, backdate)
	$recent_id = $wpdb->get_var("SELECT ID FROM $wpdb->posts WHERE post_status='publish' AND post_type='post' ORDER BY post_date DESC LIMIT 1");

	// Get a timestamp for retrieving dates later
	$date = strtotime($post->post_date);

	$args = array();
	$args['username'] = $user;
	$args['auth_method'] = 'challenge';
	$args['auth_challenge'] = $challenge;
	// Formula for challenge response is
	// md5(challenge + md5(pwd))
	$args['auth_response'] = md5($challenge . $pass);

	// Makes IJ expect UTF-8 text instead of ISO-8859-1
	$args['ver'] = "1";

	// The filters run the WP texturization - cleans up the code
	$args['event'] = $the_event;
	$args['subject'] = apply_filters('the_title', $post->post_title);

	// All of the relevent dates and times
	$args['year'] = date('Y', $date);
	$args['mon'] = date('n', $date);
	$args['day'] = date('j', $date);
	$args['hour'] = date('G', $date);
	$args['min'] = date('i', $date);
						// Enable or disable comments as specified by the
						// settings
	$args['props'] = array("opt_nocomments" => !$comments,
						// Tells IJ to not run it's formatting (replacing \n
						// with <br>, etc) because it's already been done by
						// the texturization
						"opt_preformatted" => true,
						// If the most recent post is not the one being dealt
						// with now, mark it as backdated so it doesn't jump to
						// the top of friendlists and such
						"opt_backdated" => !($post_id == $recent_id));

	// If tagging is enabled,
	if($tag) {
		// Set tags
		$args['props']['taglist'] = $cat_string;
	}

	// Set the privacy level according to the settings
	switch($privacy) {
	case "public":
		$args['security'] = "public";
		break;
	case "private":
		$args['security'] = "private";
		break;
	case "friends":
		$args['security'] = "usemask";
		$args['allowmask'] = 1;
	}

	// If crossposting to a community, specify that
	if("" != $community) {
		$args['usejournal'] = $community;
	}

	// Assume this is a new post
	$method = 'LJ.XMLRPC.postevent';

	// But check to see if there's an IJ post associated with our WP post
	if(get_post_meta($post_id, 'ijID', true)) {
		// If there is, add the itemid attribute and change from posting to editing
		$args['itemid'] = get_post_meta($post_id, 'ijID', true);
		$method = 'LJ.XMLRPC.editevent';
	}

	// And awaaaayyy we go!
	if (!$client->query($method, $args)) {
		wp_die('Something went wrong - '.$client->getErrorCode().' : '.$client->getErrorMessage());
	}

	// If we were making a new post on IJ, we need the itemid for future reference
	if('LJ.XMLRPC.postevent' == $method) {
		$response = $client->getResponse();
		// Store it to the metadata
		add_post_meta($post_id, 'ijID', $response['itemid']);
	}
	// If you don't return this, other plugins and hooks won't work
	return $post_id;
}

function ijxp_delete($post_id) {
	// Pull the post_id
	$ijxp_post_id = get_post_meta($post_id, 'ijID', true);

	// Ensures that there's actually a value. If the post was never
	// cross-posted, the value wouldn't be set, and there's no point in
	// deleting entries that don't exist
	if($ijxp_post_id == 0) {
		return $post_id;
	}

	// Get the necessary login info
	$host = get_option('ijxp_host');
	$user = get_option('ijxp_username');
	$pass = get_option('ijxp_password');

	// And open the XMLRPC interface
	$client = new IXR_Client($host, '/interface/xmlrpc');

	// Request the challenge for authentication
	if (!$client->query('LJ.XMLRPC.getchallenge')) {
		wp_die('Something went wrong - '.$client->getErrorCode().' : '.$client->getErrorMessage());
	}

	// And retrieve the challenge that IJ returns
	$response = $client->getResponse();
	$challenge = $response['challenge'];

	// Most of this is the same as before. The important difference is the
	// value of $args[event]. By setting it to a null value, IJ deletes the
	// entry. Really rather klunky way of doing things, but not my code!
	$args = array();
	$args['username'] = $user;
	$args['auth_method'] = 'challenge';
	$args['auth_challenge'] = $challenge;
	$args['auth_response'] = md5($challenge . $pass);
	$args['itemid'] = $ijxp_post_id;
	$args['event'] = "";
	$args['subject'] = "Delete this entry";
	// I probably don't need to set these, but, hell, I've got it working
	$args['year'] = date('Y');
	$args['mon'] = date('n');
	$args['day'] = date('j');
	$args['hour'] = date('G');
	$args['min'] = date('i');

	// And awaaaayyy we go!
	if (!$client->query('LJ.XMLRPC.editevent', $args)) {
		wp_die('Something went wrong - '.$client->getErrorCode().' : '.$client->getErrorMessage());
	}

	delete_post_meta($post_id, 'ijID');

	return $post_id;
}

function ijxp_edit($post_id) {
	// This function will delete a post from IJ if it's changed from the
	// published status or if crossposting was just disabled on this post

	// Pull the post_id
	$ijxp_post_id = get_post_meta($post_id, 'ijID', true);

	// Ensures that there's actually a value. If the post was never
	// cross-posted, the value wouldn't be set, so we're done
	if(0 == $ijxp_post_id) {
		return $post_id;
	}

	$post = & get_post($post_id);

	// See if the post is currently published. If it's been crossposted and its
	// state isn't published, then it should be deleted
	// Also, if it has been crossposted but it's set to not crosspost, then
	// delete it
	if('publish' != $post->post_status || 1 == get_post_meta($post_id, 'no_ij', true)) {
		ijxp_delete($post_id);
	}

	return $post_id;
}

function ijxp_sidebar() {
	global $post;
?>
	<fieldset class="dbx-box">
		<h3 class="dbx-handle"><?php _e('InsaneJournal', IJXP_DOMAIN); ?>:</h3>
		<div class="dbx-content">
			<label for="ijxp_crosspost_go" class="selectit">
			<input id="ijxp_crosspost_go" type="radio" name="ijxp_crosspost" value="1"<?php checked(get_post_meta($post->ID, 'no_ij', true), 0); ?>/>
			<?php _e('Crosspost', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_crosspost_nogo" class="selectit">
			<input id="ijxp_crosspost_nogo" type="radio" name="ijxp_crosspost" value="0"<?php checked(get_post_meta($post->ID, 'no_ij', true), 1); ?>/>
			<?php _e('Do not crosspost', IJXP_DOMAIN); ?>
			</label>

			<br />

			<label for="ijxp_comments_default" class="selectit">
			<input id="ijxp_comments_default" type="radio" name="ijxp_comments" value="0"<?php checked(get_post_meta($post->ID, 'ijxp_comments', true), 0); ?>/>
			<?php _e('Default comments setting', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_comments_on" class="selectit">
			<input id="ijxp_comments_on" type="radio" name="ijxp_comments" value="1"<?php checked(get_post_meta($post->ID, 'ijxp_comments', true), 1); ?>/>
			<?php _e('Comments on', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_comments_off" class="selectit">
			<input id="ijxp_comments_off" type="radio" name="ijxp_comments" value="2"<?php checked(get_post_meta($post->ID, 'ijxp_comments', true), 2); ?>/>
			<?php _e('Comments off', IJXP_DOMAIN); ?>
			</label>

			<br />

			<label for="ijxp_privacy_default" class="selectit">
			<input id="ijxp_privacy_default" type="radio" name="ijxp_privacy" value="0"<?php checked(get_post_meta($post->ID, 'ijxp_privacy', true), 0); ?>/>
			<?php _e('Default post privacy setting', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_privacy_public" class="selectit">
			<input id="ijxp_privacy_public" type="radio" name="ijxp_privacy" value="public"<?php checked(get_post_meta($post->ID, 'ijxp_privacy', true), 'public'); ?>/>
			<?php _e('Public post', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_privacy_private" class="selectit">
			<input id="ijxp_privacy_private" type="radio" name="ijxp_privacy" value="private"<?php checked(get_post_meta($post->ID, 'ijxp_privacy', true), 'private'); ?>/>
			<?php _e('Private post', IJXP_DOMAIN); ?>
			</label>
			<label for="ijxp_privacy_friends" class="selectit">
			<input id="ijxp_privacy_friends" type="radio" name="ijxp_privacy" value="friends"<?php checked(get_post_meta($post->ID, 'ijxp_privacy', true), 'friends'); ?>/>
			<?php _e('Friends only', IJXP_DOMAIN); ?>
			</label>
		</div>
	</fieldset>
<?php
}

function ijxp_save($post_id) {
	// If the magic crossposting variable isn't equal to 'crosspost', then the
	// box wasn't checked
	// Using publish_post hook for the case of a state change---this will
	// be called before crossposting occurs
	// Using save_post for the case where it's draft or private - the value
	// still needs to be saved
	// Using edit_post for the case in which it's changed from crossposted to
	// not crossposted in an edit

	// At least one of those hooks is probably unnecessary, but I can't figure
	// out which one
	if(isset($_POST['ijxp_crosspost'])) {
		delete_post_meta($post_id, 'no_ij');
		if(0 == $_POST['ijxp_crosspost']) {
			add_post_meta($post_id, 'no_ij', '1');
		}
	}
	if(isset($_POST['ijxp_comments'])) {
		delete_post_meta($post_id, 'ijxp_comments');
		if("0" != $_POST['ijxp_comments']) {
			add_post_meta($post_id, 'ijxp_comments', $_POST['ijxp_comments']);
		}
	}
	if(isset($_POST['ijxp_privacy'])) {
		delete_post_meta($post_id, 'ijxp_privacy');
		if("0" != $_POST['ijxp_privacy']) {
			add_post_meta($post_id, 'ijxp_privacy', $_POST['ijxp_privacy']);
		}
	}
}

function ijxp_delete_all($repost_ids) {
	foreach((array)$repost_ids as $id) {
		ijxp_delete($id);
	}
}

function ijxp_post_all($repost_ids) {
	foreach((array)$repost_ids as $id) {
		ijxp_post($id);
	}
}

add_action('admin_menu', 'ijxp_add_pages');
if(get_option('ijxp_username') != "") {
	add_action('publish_post', 'ijxp_post');
	add_action('edit_post', 'ijxp_edit');
	add_action('delete_post', 'ijxp_delete');
	add_action('dbx_post_sidebar', 'ijxp_sidebar');
	add_action('publish_post', 'ijxp_save', 1);
	add_action('save_post', 'ijxp_save', 1);
	add_action('edit_post', 'ijxp_save', 1);
}

?>